config = {
    "interfaces": {
        "google.privacy.dlp.v2.DlpService": {
            "retry_codes": {
                "idempotent": ["DEADLINE_EXCEEDED", "UNAVAILABLE"],
                "non_idempotent": [],
            },
            "retry_params": {
                "default": {
                    "initial_retry_delay_millis": 100,
                    "retry_delay_multiplier": 1.3,
                    "max_retry_delay_millis": 60000,
                    "initial_rpc_timeout_millis": 20000,
                    "rpc_timeout_multiplier": 1.0,
                    "max_rpc_timeout_millis": 20000,
                    "total_timeout_millis": 600000,
                }
            },
            "methods": {
                "InspectContent": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "RedactImage": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeidentifyContent": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "ReidentifyContent": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "ListInfoTypes": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "CreateInspectTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "UpdateInspectTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "GetInspectTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "ListInspectTemplates": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeleteInspectTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "CreateDeidentifyTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "UpdateDeidentifyTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "GetDeidentifyTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "ListDeidentifyTemplates": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeleteDeidentifyTemplate": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "CreateDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "ListDlpJobs": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "GetDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeleteDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "CancelDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "FinishDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "HybridInspectDlpJob": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "ListJobTriggers": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "GetJobTrigger": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeleteJobTrigger": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "HybridInspectJobTrigger": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "UpdateJobTrigger": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "CreateJobTrigger": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "CreateStoredInfoType": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "UpdateStoredInfoType": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "non_idempotent",
                    "retry_params_name": "default",
                },
                "GetStoredInfoType": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "ListStoredInfoTypes": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
                "DeleteStoredInfoType": {
                    "timeout_millis": 300000,
                    "retry_codes_name": "idempotent",
                    "retry_params_name": "default",
                },
            },
        }
    }
}



# -*- coding: utf-8 -*-
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import google.api_core.grpc_helpers

from google.cloud.dlp_v2.proto import dlp_pb2_grpc


class DlpServiceGrpcTransport(object):
    """gRPC transport class providing stubs for
    google.privacy.dlp.v2 DlpService API.

    The transport provides access to the raw gRPC stubs,
    which can be used to take advantage of advanced
    features of gRPC.
    """

    # The scopes needed to make gRPC calls to all of the methods defined
    # in this service.
    _OAUTH_SCOPES = ("https://www.googleapis.com/auth/cloud-platform",)

    def __init__(
        self, channel=None, credentials=None, address="dlp.googleapis.com:443"
    ):
        """Instantiate the transport class.

        Args:
            channel (grpc.Channel): A ``Channel`` instance through
                which to make calls. This argument is mutually exclusive
                with ``credentials``; providing both will raise an exception.
            credentials (google.auth.credentials.Credentials): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            address (str): The address where the service is hosted.
        """
        # If both `channel` and `credentials` are specified, raise an
        # exception (channels come with credentials baked in already).
        if channel is not None and credentials is not None:
            raise ValueError(
                "The `channel` and `credentials` arguments are mutually " "exclusive."
            )

        # Create the channel.
        if channel is None:
            channel = self.create_channel(
                address=address,
                credentials=credentials,
                options={
                    "grpc.max_send_message_length": -1,
                    "grpc.max_receive_message_length": -1,
                }.items(),
            )

        self._channel = channel

        # gRPC uses objects called "stubs" that are bound to the
        # channel and provide a basic method for each RPC.
        self._stubs = {"dlp_service_stub": dlp_pb2_grpc.DlpServiceStub(channel)}

    @classmethod
    def create_channel(
        cls, address="dlp.googleapis.com:443", credentials=None, **kwargs
    ):
        """Create and return a gRPC channel object.

        Args:
            address (str): The host for the channel to use.
            credentials (~.Credentials): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            kwargs (dict): Keyword arguments, which are passed to the
                channel creation.

        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return google.api_core.grpc_helpers.create_channel(
            address, credentials=credentials, scopes=cls._OAUTH_SCOPES, **kwargs
        )

    @property
    def channel(self):
        """The gRPC channel used by the transport.

        Returns:
            grpc.Channel: A gRPC channel object.
        """
        return self._channel

    @property
    def inspect_content(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.inspect_content`.

        Finds potentially sensitive info in content.
        This method has limits on input size, processing time, and output size.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.

        For how to guides, see https://cloud.google.com/dlp/docs/inspecting-images
        and https://cloud.google.com/dlp/docs/inspecting-text,

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].InspectContent

    @property
    def redact_image(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.redact_image`.

        Redacts potentially sensitive info from an image.
        This method has limits on input size, processing time, and output size.
        See https://cloud.google.com/dlp/docs/redacting-sensitive-data-images to
        learn more.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].RedactImage

    @property
    def deidentify_content(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.deidentify_content`.

        De-identifies potentially sensitive info from a ContentItem.
        This method has limits on input size and output size.
        See https://cloud.google.com/dlp/docs/deidentify-sensitive-data to
        learn more.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeidentifyContent

    @property
    def reidentify_content(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.reidentify_content`.

        Re-identifies content that has been de-identified. See
        https://cloud.google.com/dlp/docs/pseudonymization#re-identification_in_free_text_code_example
        to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ReidentifyContent

    @property
    def list_info_types(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_info_types`.

        Returns a list of the sensitive information types that the DLP API
        supports. See https://cloud.google.com/dlp/docs/infotypes-reference to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListInfoTypes

    @property
    def create_inspect_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.create_inspect_template`.

        Creates an InspectTemplate for re-using frequently used configuration
        for inspecting content, images, and storage.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CreateInspectTemplate

    @property
    def update_inspect_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.update_inspect_template`.

        Updates the InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].UpdateInspectTemplate

    @property
    def get_inspect_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.get_inspect_template`.

        Gets an InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].GetInspectTemplate

    @property
    def list_inspect_templates(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_inspect_templates`.

        Lists InspectTemplates.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListInspectTemplates

    @property
    def delete_inspect_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.delete_inspect_template`.

        Deletes an InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeleteInspectTemplate

    @property
    def create_deidentify_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.create_deidentify_template`.

        Creates a DeidentifyTemplate for re-using frequently used configuration
        for de-identifying content, images, and storage.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CreateDeidentifyTemplate

    @property
    def update_deidentify_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.update_deidentify_template`.

        Updates the DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].UpdateDeidentifyTemplate

    @property
    def get_deidentify_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.get_deidentify_template`.

        Gets a DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].GetDeidentifyTemplate

    @property
    def list_deidentify_templates(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_deidentify_templates`.

        Lists DeidentifyTemplates.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListDeidentifyTemplates

    @property
    def delete_deidentify_template(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.delete_deidentify_template`.

        Deletes a DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeleteDeidentifyTemplate

    @property
    def create_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.create_dlp_job`.

        Creates a new job to inspect storage or calculate risk metrics.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        When no InfoTypes or CustomInfoTypes are specified in inspect jobs, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CreateDlpJob

    @property
    def list_dlp_jobs(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_dlp_jobs`.

        Lists DlpJobs that match the specified filter in the request.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListDlpJobs

    @property
    def get_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.get_dlp_job`.

        Gets the latest state of a long-running DlpJob.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].GetDlpJob

    @property
    def delete_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.delete_dlp_job`.

        Deletes a long-running DlpJob. This method indicates that the client is
        no longer interested in the DlpJob result. The job will be cancelled if
        possible.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeleteDlpJob

    @property
    def cancel_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.cancel_dlp_job`.

        Starts asynchronous cancellation on a long-running DlpJob. The server
        makes a best effort to cancel the DlpJob, but success is not
        guaranteed.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CancelDlpJob

    @property
    def finish_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.finish_dlp_job`.

        Finish a running hybrid DlpJob. Triggers the finalization steps and running
        of any enabled actions that have not yet run.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].FinishDlpJob

    @property
    def hybrid_inspect_dlp_job(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.hybrid_inspect_dlp_job`.

        Inspect hybrid content and store findings to a job.
        To review the findings inspect the job. Inspection will occur
        asynchronously.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].HybridInspectDlpJob

    @property
    def list_job_triggers(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_job_triggers`.

        Lists job triggers.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListJobTriggers

    @property
    def get_job_trigger(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.get_job_trigger`.

        Gets a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].GetJobTrigger

    @property
    def delete_job_trigger(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.delete_job_trigger`.

        Deletes a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeleteJobTrigger

    @property
    def hybrid_inspect_job_trigger(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.hybrid_inspect_job_trigger`.

        Inspect hybrid content and store findings to a trigger. The inspection
        will be processed asynchronously. To review the findings monitor the
        jobs within the trigger.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].HybridInspectJobTrigger

    @property
    def update_job_trigger(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.update_job_trigger`.

        Updates a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].UpdateJobTrigger

    @property
    def create_job_trigger(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.create_job_trigger`.

        Creates a job trigger to run DLP actions such as scanning storage for
        sensitive information on a set schedule.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CreateJobTrigger

    @property
    def create_stored_info_type(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.create_stored_info_type`.

        Creates a pre-built stored infoType to be used for inspection.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].CreateStoredInfoType

    @property
    def update_stored_info_type(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.update_stored_info_type`.

        Updates the stored infoType by creating a new version. The existing version
        will continue to be used until the new version is ready.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].UpdateStoredInfoType

    @property
    def get_stored_info_type(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.get_stored_info_type`.

        Gets a stored infoType.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].GetStoredInfoType

    @property
    def list_stored_info_types(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.list_stored_info_types`.

        Lists stored infoTypes.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].ListStoredInfoTypes

    @property
    def delete_stored_info_type(self):
        """Return the gRPC stub for :meth:`DlpServiceClient.delete_stored_info_type`.

        Deletes a stored infoType.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.

        Returns:
            Callable: A callable which accepts the appropriate
                deserialized request object and returns a
                deserialized response object.
        """
        return self._stubs["dlp_service_stub"].DeleteStoredInfoType



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from google.cloud.dlp_v2.proto import (
    dlp_pb2 as google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2,
)
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class DlpServiceStub(object):
    """The Cloud Data Loss Prevention (DLP) API is a service that allows clients
    to detect the presence of Personally Identifiable Information (PII) and other
    privacy-sensitive data in user-supplied, unstructured data streams, like text
    blocks or images.
    The service also includes methods for sensitive data redaction and
    scheduling of data scans on Google Cloud Platform based data sets.

    To learn more about concepts and find how-to guides see
    https://cloud.google.com/dlp/docs/.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
          channel: A grpc.Channel.
        """
        self.InspectContent = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/InspectContent",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectContentRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectContentResponse.FromString,
        )
        self.RedactImage = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/RedactImage",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.RedactImageRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.RedactImageResponse.FromString,
        )
        self.DeidentifyContent = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeidentifyContent",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyContentRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyContentResponse.FromString,
        )
        self.ReidentifyContent = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ReidentifyContent",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ReidentifyContentRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ReidentifyContentResponse.FromString,
        )
        self.ListInfoTypes = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListInfoTypes",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInfoTypesRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInfoTypesResponse.FromString,
        )
        self.CreateInspectTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CreateInspectTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateInspectTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.FromString,
        )
        self.UpdateInspectTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/UpdateInspectTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateInspectTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.FromString,
        )
        self.GetInspectTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/GetInspectTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetInspectTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.FromString,
        )
        self.ListInspectTemplates = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListInspectTemplates",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInspectTemplatesRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInspectTemplatesResponse.FromString,
        )
        self.DeleteInspectTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeleteInspectTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteInspectTemplateRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CreateDeidentifyTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CreateDeidentifyTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateDeidentifyTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.FromString,
        )
        self.UpdateDeidentifyTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/UpdateDeidentifyTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateDeidentifyTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.FromString,
        )
        self.GetDeidentifyTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/GetDeidentifyTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetDeidentifyTemplateRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.FromString,
        )
        self.ListDeidentifyTemplates = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListDeidentifyTemplates",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDeidentifyTemplatesRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDeidentifyTemplatesResponse.FromString,
        )
        self.DeleteDeidentifyTemplate = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeleteDeidentifyTemplate",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteDeidentifyTemplateRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CreateJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CreateJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.FromString,
        )
        self.UpdateJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/UpdateJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.FromString,
        )
        self.HybridInspectJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/HybridInspectJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectResponse.FromString,
        )
        self.GetJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/GetJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.FromString,
        )
        self.ListJobTriggers = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListJobTriggers",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListJobTriggersRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListJobTriggersResponse.FromString,
        )
        self.DeleteJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeleteJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.ActivateJobTrigger = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ActivateJobTrigger",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ActivateJobTriggerRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.FromString,
        )
        self.CreateDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CreateDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.FromString,
        )
        self.ListDlpJobs = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListDlpJobs",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDlpJobsRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDlpJobsResponse.FromString,
        )
        self.GetDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/GetDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.FromString,
        )
        self.DeleteDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeleteDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CancelDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CancelDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CancelDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.CreateStoredInfoType = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/CreateStoredInfoType",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateStoredInfoTypeRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.FromString,
        )
        self.UpdateStoredInfoType = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/UpdateStoredInfoType",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateStoredInfoTypeRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.FromString,
        )
        self.GetStoredInfoType = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/GetStoredInfoType",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetStoredInfoTypeRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.FromString,
        )
        self.ListStoredInfoTypes = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/ListStoredInfoTypes",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListStoredInfoTypesRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListStoredInfoTypesResponse.FromString,
        )
        self.DeleteStoredInfoType = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/DeleteStoredInfoType",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteStoredInfoTypeRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
        self.HybridInspectDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/HybridInspectDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectResponse.FromString,
        )
        self.FinishDlpJob = channel.unary_unary(
            "/google.privacy.dlp.v2.DlpService/FinishDlpJob",
            request_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.FinishDlpJobRequest.SerializeToString,
            response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class DlpServiceServicer(object):
    """The Cloud Data Loss Prevention (DLP) API is a service that allows clients
    to detect the presence of Personally Identifiable Information (PII) and other
    privacy-sensitive data in user-supplied, unstructured data streams, like text
    blocks or images.
    The service also includes methods for sensitive data redaction and
    scheduling of data scans on Google Cloud Platform based data sets.

    To learn more about concepts and find how-to guides see
    https://cloud.google.com/dlp/docs/.
    """

    def InspectContent(self, request, context):
        """Finds potentially sensitive info in content.
        This method has limits on input size, processing time, and output size.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.

        For how to guides, see https://cloud.google.com/dlp/docs/inspecting-images
        and https://cloud.google.com/dlp/docs/inspecting-text,
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RedactImage(self, request, context):
        """Redacts potentially sensitive info from an image.
        This method has limits on input size, processing time, and output size.
        See https://cloud.google.com/dlp/docs/redacting-sensitive-data-images to
        learn more.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeidentifyContent(self, request, context):
        """De-identifies potentially sensitive info from a ContentItem.
        This method has limits on input size and output size.
        See https://cloud.google.com/dlp/docs/deidentify-sensitive-data to
        learn more.

        When no InfoTypes or CustomInfoTypes are specified in this request, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ReidentifyContent(self, request, context):
        """Re-identifies content that has been de-identified.
        See
        https://cloud.google.com/dlp/docs/pseudonymization#re-identification_in_free_text_code_example
        to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListInfoTypes(self, request, context):
        """Returns a list of the sensitive information types that the DLP API
        supports. See https://cloud.google.com/dlp/docs/infotypes-reference to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateInspectTemplate(self, request, context):
        """Creates an InspectTemplate for re-using frequently used configuration
        for inspecting content, images, and storage.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateInspectTemplate(self, request, context):
        """Updates the InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetInspectTemplate(self, request, context):
        """Gets an InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListInspectTemplates(self, request, context):
        """Lists InspectTemplates.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteInspectTemplate(self, request, context):
        """Deletes an InspectTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateDeidentifyTemplate(self, request, context):
        """Creates a DeidentifyTemplate for re-using frequently used configuration
        for de-identifying content, images, and storage.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateDeidentifyTemplate(self, request, context):
        """Updates the DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetDeidentifyTemplate(self, request, context):
        """Gets a DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListDeidentifyTemplates(self, request, context):
        """Lists DeidentifyTemplates.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteDeidentifyTemplate(self, request, context):
        """Deletes a DeidentifyTemplate.
        See https://cloud.google.com/dlp/docs/creating-templates-deid to learn
        more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateJobTrigger(self, request, context):
        """Creates a job trigger to run DLP actions such as scanning storage for
        sensitive information on a set schedule.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateJobTrigger(self, request, context):
        """Updates a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def HybridInspectJobTrigger(self, request, context):
        """Inspect hybrid content and store findings to a trigger. The inspection
        will be processed asynchronously. To review the findings monitor the
        jobs within the trigger.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetJobTrigger(self, request, context):
        """Gets a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListJobTriggers(self, request, context):
        """Lists job triggers.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteJobTrigger(self, request, context):
        """Deletes a job trigger.
        See https://cloud.google.com/dlp/docs/creating-job-triggers to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ActivateJobTrigger(self, request, context):
        """Activate a job trigger. Causes the immediate execute of a trigger
        instead of waiting on the trigger event to occur.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateDlpJob(self, request, context):
        """Creates a new job to inspect storage or calculate risk metrics.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.

        When no InfoTypes or CustomInfoTypes are specified in inspect jobs, the
        system will automatically choose what detectors to run. By default this may
        be all types, but may change over time as detectors are updated.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListDlpJobs(self, request, context):
        """Lists DlpJobs that match the specified filter in the request.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetDlpJob(self, request, context):
        """Gets the latest state of a long-running DlpJob.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteDlpJob(self, request, context):
        """Deletes a long-running DlpJob. This method indicates that the client is
        no longer interested in the DlpJob result. The job will be cancelled if
        possible.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CancelDlpJob(self, request, context):
        """Starts asynchronous cancellation on a long-running DlpJob. The server
        makes a best effort to cancel the DlpJob, but success is not
        guaranteed.
        See https://cloud.google.com/dlp/docs/inspecting-storage and
        https://cloud.google.com/dlp/docs/compute-risk-analysis to learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateStoredInfoType(self, request, context):
        """Creates a pre-built stored infoType to be used for inspection.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateStoredInfoType(self, request, context):
        """Updates the stored infoType by creating a new version. The existing version
        will continue to be used until the new version is ready.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetStoredInfoType(self, request, context):
        """Gets a stored infoType.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListStoredInfoTypes(self, request, context):
        """Lists stored infoTypes.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteStoredInfoType(self, request, context):
        """Deletes a stored infoType.
        See https://cloud.google.com/dlp/docs/creating-stored-infotypes to
        learn more.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def HybridInspectDlpJob(self, request, context):
        """Inspect hybrid content and store findings to a job.
        To review the findings inspect the job. Inspection will occur
        asynchronously.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def FinishDlpJob(self, request, context):
        """Finish a running hybrid DlpJob. Triggers the finalization steps and running
        of any enabled actions that have not yet run.
        Early access feature is in a pre-release state and might change or have
        limited support. For more information, see
        https://cloud.google.com/products#product-launch-stages.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_DlpServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "InspectContent": grpc.unary_unary_rpc_method_handler(
            servicer.InspectContent,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectContentRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectContentResponse.SerializeToString,
        ),
        "RedactImage": grpc.unary_unary_rpc_method_handler(
            servicer.RedactImage,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.RedactImageRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.RedactImageResponse.SerializeToString,
        ),
        "DeidentifyContent": grpc.unary_unary_rpc_method_handler(
            servicer.DeidentifyContent,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyContentRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyContentResponse.SerializeToString,
        ),
        "ReidentifyContent": grpc.unary_unary_rpc_method_handler(
            servicer.ReidentifyContent,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ReidentifyContentRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ReidentifyContentResponse.SerializeToString,
        ),
        "ListInfoTypes": grpc.unary_unary_rpc_method_handler(
            servicer.ListInfoTypes,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInfoTypesRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInfoTypesResponse.SerializeToString,
        ),
        "CreateInspectTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.CreateInspectTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateInspectTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.SerializeToString,
        ),
        "UpdateInspectTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateInspectTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateInspectTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.SerializeToString,
        ),
        "GetInspectTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.GetInspectTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetInspectTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.InspectTemplate.SerializeToString,
        ),
        "ListInspectTemplates": grpc.unary_unary_rpc_method_handler(
            servicer.ListInspectTemplates,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInspectTemplatesRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListInspectTemplatesResponse.SerializeToString,
        ),
        "DeleteInspectTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteInspectTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteInspectTemplateRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CreateDeidentifyTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.CreateDeidentifyTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateDeidentifyTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.SerializeToString,
        ),
        "UpdateDeidentifyTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateDeidentifyTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateDeidentifyTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.SerializeToString,
        ),
        "GetDeidentifyTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.GetDeidentifyTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetDeidentifyTemplateRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeidentifyTemplate.SerializeToString,
        ),
        "ListDeidentifyTemplates": grpc.unary_unary_rpc_method_handler(
            servicer.ListDeidentifyTemplates,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDeidentifyTemplatesRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDeidentifyTemplatesResponse.SerializeToString,
        ),
        "DeleteDeidentifyTemplate": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteDeidentifyTemplate,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteDeidentifyTemplateRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CreateJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.CreateJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateJobTriggerRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.SerializeToString,
        ),
        "UpdateJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateJobTriggerRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.SerializeToString,
        ),
        "HybridInspectJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.HybridInspectJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectJobTriggerRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectResponse.SerializeToString,
        ),
        "GetJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.GetJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetJobTriggerRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.JobTrigger.SerializeToString,
        ),
        "ListJobTriggers": grpc.unary_unary_rpc_method_handler(
            servicer.ListJobTriggers,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListJobTriggersRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListJobTriggersResponse.SerializeToString,
        ),
        "DeleteJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteJobTriggerRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "ActivateJobTrigger": grpc.unary_unary_rpc_method_handler(
            servicer.ActivateJobTrigger,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ActivateJobTriggerRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.SerializeToString,
        ),
        "CreateDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.CreateDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateDlpJobRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.SerializeToString,
        ),
        "ListDlpJobs": grpc.unary_unary_rpc_method_handler(
            servicer.ListDlpJobs,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDlpJobsRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListDlpJobsResponse.SerializeToString,
        ),
        "GetDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.GetDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetDlpJobRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DlpJob.SerializeToString,
        ),
        "DeleteDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteDlpJobRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CancelDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.CancelDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CancelDlpJobRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "CreateStoredInfoType": grpc.unary_unary_rpc_method_handler(
            servicer.CreateStoredInfoType,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.CreateStoredInfoTypeRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.SerializeToString,
        ),
        "UpdateStoredInfoType": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateStoredInfoType,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.UpdateStoredInfoTypeRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.SerializeToString,
        ),
        "GetStoredInfoType": grpc.unary_unary_rpc_method_handler(
            servicer.GetStoredInfoType,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.GetStoredInfoTypeRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.StoredInfoType.SerializeToString,
        ),
        "ListStoredInfoTypes": grpc.unary_unary_rpc_method_handler(
            servicer.ListStoredInfoTypes,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListStoredInfoTypesRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.ListStoredInfoTypesResponse.SerializeToString,
        ),
        "DeleteStoredInfoType": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteStoredInfoType,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.DeleteStoredInfoTypeRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
        "HybridInspectDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.HybridInspectDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectDlpJobRequest.FromString,
            response_serializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.HybridInspectResponse.SerializeToString,
        ),
        "FinishDlpJob": grpc.unary_unary_rpc_method_handler(
            servicer.FinishDlpJob,
            request_deserializer=google_dot_cloud_dot_dlp__v2_dot_proto_dot_dlp__pb2.FinishDlpJobRequest.FromString,
            response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "google.privacy.dlp.v2.DlpService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unit tests."""

import mock
import pytest

from google.cloud import dlp_v2
from google.cloud.dlp_v2.proto import dlp_pb2
from google.protobuf import empty_pb2


class MultiCallableStub(object):
    """Stub for the grpc.UnaryUnaryMultiCallable interface."""

    def __init__(self, method, channel_stub):
        self.method = method
        self.channel_stub = channel_stub

    def __call__(self, request, timeout=None, metadata=None, credentials=None):
        self.channel_stub.requests.append((self.method, request))

        response = None
        if self.channel_stub.responses:
            response = self.channel_stub.responses.pop()

        if isinstance(response, Exception):
            raise response

        if response:
            return response


class ChannelStub(object):
    """Stub for the grpc.Channel interface."""

    def __init__(self, responses=[]):
        self.responses = responses
        self.requests = []

    def unary_unary(self, method, request_serializer=None, response_deserializer=None):
        return MultiCallableStub(method, self)


class CustomException(Exception):
    pass


class TestDlpServiceClient(object):
    def test_inspect_content(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.InspectContentResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        response = client.inspect_content()
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.InspectContentRequest()
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_inspect_content_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        with pytest.raises(CustomException):
            client.inspect_content()

    def test_redact_image(self):
        # Setup Expected Response
        redacted_image = b"28"
        extracted_text = "extractedText998260012"
        expected_response = {
            "redacted_image": redacted_image,
            "extracted_text": extracted_text,
        }
        expected_response = dlp_pb2.RedactImageResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        response = client.redact_image()
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.RedactImageRequest()
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_redact_image_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        with pytest.raises(CustomException):
            client.redact_image()

    def test_deidentify_content(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.DeidentifyContentResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        response = client.deidentify_content()
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeidentifyContentRequest()
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_deidentify_content_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        with pytest.raises(CustomException):
            client.deidentify_content()

    def test_reidentify_content(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.ReidentifyContentResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.project_path("[PROJECT]")

        response = client.reidentify_content(parent)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ReidentifyContentRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_reidentify_content_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.project_path("[PROJECT]")

        with pytest.raises(CustomException):
            client.reidentify_content(parent)

    def test_list_info_types(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.ListInfoTypesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        response = client.list_info_types()
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListInfoTypesRequest()
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_info_types_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        with pytest.raises(CustomException):
            client.list_info_types()

    def test_create_inspect_template(self):
        # Setup Expected Response
        name = "name3373707"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.InspectTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        inspect_template = {}

        response = client.create_inspect_template(parent, inspect_template)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CreateInspectTemplateRequest(
            parent=parent, inspect_template=inspect_template
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_inspect_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        inspect_template = {}

        with pytest.raises(CustomException):
            client.create_inspect_template(parent, inspect_template)

    def test_update_inspect_template(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.InspectTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        response = client.update_inspect_template(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.UpdateInspectTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_inspect_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.update_inspect_template(name)

    def test_get_inspect_template(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.InspectTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        response = client.get_inspect_template(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.GetInspectTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_inspect_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.get_inspect_template(name)

    def test_list_inspect_templates(self):
        # Setup Expected Response
        next_page_token = ""
        inspect_templates_element = {}
        inspect_templates = [inspect_templates_element]
        expected_response = {
            "next_page_token": next_page_token,
            "inspect_templates": inspect_templates,
        }
        expected_response = dlp_pb2.ListInspectTemplatesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_inspect_templates(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.inspect_templates[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListInspectTemplatesRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_inspect_templates_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_inspect_templates(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_delete_inspect_template(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        client.delete_inspect_template(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeleteInspectTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_inspect_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_inspect_template_path(
            "[ORGANIZATION]", "[INSPECT_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.delete_inspect_template(name)

    def test_create_deidentify_template(self):
        # Setup Expected Response
        name = "name3373707"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.DeidentifyTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        deidentify_template = {}

        response = client.create_deidentify_template(parent, deidentify_template)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CreateDeidentifyTemplateRequest(
            parent=parent, deidentify_template=deidentify_template
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_deidentify_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        deidentify_template = {}

        with pytest.raises(CustomException):
            client.create_deidentify_template(parent, deidentify_template)

    def test_update_deidentify_template(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.DeidentifyTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        response = client.update_deidentify_template(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.UpdateDeidentifyTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_deidentify_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.update_deidentify_template(name)

    def test_get_deidentify_template(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.DeidentifyTemplate(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        response = client.get_deidentify_template(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.GetDeidentifyTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_deidentify_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.get_deidentify_template(name)

    def test_list_deidentify_templates(self):
        # Setup Expected Response
        next_page_token = ""
        deidentify_templates_element = {}
        deidentify_templates = [deidentify_templates_element]
        expected_response = {
            "next_page_token": next_page_token,
            "deidentify_templates": deidentify_templates,
        }
        expected_response = dlp_pb2.ListDeidentifyTemplatesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_deidentify_templates(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.deidentify_templates[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListDeidentifyTemplatesRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_deidentify_templates_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_deidentify_templates(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_delete_deidentify_template(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        client.delete_deidentify_template(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeleteDeidentifyTemplateRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_deidentify_template_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_deidentify_template_path(
            "[ORGANIZATION]", "[DEIDENTIFY_TEMPLATE]"
        )

        with pytest.raises(CustomException):
            client.delete_deidentify_template(name)

    def test_create_dlp_job(self):
        # Setup Expected Response
        name = "name3373707"
        job_trigger_name = "jobTriggerName1819490804"
        expected_response = {"name": name, "job_trigger_name": job_trigger_name}
        expected_response = dlp_pb2.DlpJob(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.project_path("[PROJECT]")

        response = client.create_dlp_job(parent)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CreateDlpJobRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.project_path("[PROJECT]")

        with pytest.raises(CustomException):
            client.create_dlp_job(parent)

    def test_list_dlp_jobs(self):
        # Setup Expected Response
        next_page_token = ""
        jobs_element = {}
        jobs = [jobs_element]
        expected_response = {"next_page_token": next_page_token, "jobs": jobs}
        expected_response = dlp_pb2.ListDlpJobsResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_dlp_jobs(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.jobs[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListDlpJobsRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_dlp_jobs_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_dlp_jobs(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_get_dlp_job(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        job_trigger_name = "jobTriggerName1819490804"
        expected_response = {"name": name_2, "job_trigger_name": job_trigger_name}
        expected_response = dlp_pb2.DlpJob(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = "name3373707"

        response = client.get_dlp_job(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.GetDlpJobRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = "name3373707"

        with pytest.raises(CustomException):
            client.get_dlp_job(name)

    def test_delete_dlp_job(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = "name3373707"

        client.delete_dlp_job(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeleteDlpJobRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = "name3373707"

        with pytest.raises(CustomException):
            client.delete_dlp_job(name)

    def test_cancel_dlp_job(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = "name3373707"

        client.cancel_dlp_job(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CancelDlpJobRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_cancel_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = "name3373707"

        with pytest.raises(CustomException):
            client.cancel_dlp_job(name)

    def test_finish_dlp_job(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = "name3373707"

        client.finish_dlp_job(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.FinishDlpJobRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_finish_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = "name3373707"

        with pytest.raises(CustomException):
            client.finish_dlp_job(name)

    def test_hybrid_inspect_dlp_job(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.HybridInspectResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = "name3373707"

        response = client.hybrid_inspect_dlp_job(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.HybridInspectDlpJobRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_hybrid_inspect_dlp_job_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = "name3373707"

        with pytest.raises(CustomException):
            client.hybrid_inspect_dlp_job(name)

    def test_list_job_triggers(self):
        # Setup Expected Response
        next_page_token = ""
        job_triggers_element = {}
        job_triggers = [job_triggers_element]
        expected_response = {
            "next_page_token": next_page_token,
            "job_triggers": job_triggers,
        }
        expected_response = dlp_pb2.ListJobTriggersResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_job_triggers(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.job_triggers[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListJobTriggersRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_job_triggers_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.project_path("[PROJECT]")

        paged_list_response = client.list_job_triggers(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_get_job_trigger(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.JobTrigger(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        response = client.get_job_trigger(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.GetJobTriggerRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_job_trigger_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        with pytest.raises(CustomException):
            client.get_job_trigger(name)

    def test_delete_job_trigger(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        client.delete_job_trigger(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeleteJobTriggerRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_job_trigger_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        with pytest.raises(CustomException):
            client.delete_job_trigger(name)

    def test_hybrid_inspect_job_trigger(self):
        # Setup Expected Response
        expected_response = {}
        expected_response = dlp_pb2.HybridInspectResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        response = client.hybrid_inspect_job_trigger(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.HybridInspectJobTriggerRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_hybrid_inspect_job_trigger_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        with pytest.raises(CustomException):
            client.hybrid_inspect_job_trigger(name)

    def test_update_job_trigger(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name_2,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.JobTrigger(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        response = client.update_job_trigger(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.UpdateJobTriggerRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_job_trigger_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.project_job_trigger_path("[PROJECT]", "[JOB_TRIGGER]")

        with pytest.raises(CustomException):
            client.update_job_trigger(name)

    def test_create_job_trigger(self):
        # Setup Expected Response
        name = "name3373707"
        display_name = "displayName1615086568"
        description = "description-1724546052"
        expected_response = {
            "name": name,
            "display_name": display_name,
            "description": description,
        }
        expected_response = dlp_pb2.JobTrigger(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.project_path("[PROJECT]")
        job_trigger = {}

        response = client.create_job_trigger(parent, job_trigger)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CreateJobTriggerRequest(
            parent=parent, job_trigger=job_trigger
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_job_trigger_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.project_path("[PROJECT]")
        job_trigger = {}

        with pytest.raises(CustomException):
            client.create_job_trigger(parent, job_trigger)

    def test_create_stored_info_type(self):
        # Setup Expected Response
        name = "name3373707"
        expected_response = {"name": name}
        expected_response = dlp_pb2.StoredInfoType(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        config = {}

        response = client.create_stored_info_type(parent, config)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.CreateStoredInfoTypeRequest(
            parent=parent, config=config
        )
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_create_stored_info_type_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")
        config = {}

        with pytest.raises(CustomException):
            client.create_stored_info_type(parent, config)

    def test_update_stored_info_type(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        expected_response = {"name": name_2}
        expected_response = dlp_pb2.StoredInfoType(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        response = client.update_stored_info_type(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.UpdateStoredInfoTypeRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_update_stored_info_type_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        with pytest.raises(CustomException):
            client.update_stored_info_type(name)

    def test_get_stored_info_type(self):
        # Setup Expected Response
        name_2 = "name2-1052831874"
        expected_response = {"name": name_2}
        expected_response = dlp_pb2.StoredInfoType(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        response = client.get_stored_info_type(name)
        assert expected_response == response

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.GetStoredInfoTypeRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_get_stored_info_type_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        with pytest.raises(CustomException):
            client.get_stored_info_type(name)

    def test_list_stored_info_types(self):
        # Setup Expected Response
        next_page_token = ""
        stored_info_types_element = {}
        stored_info_types = [stored_info_types_element]
        expected_response = {
            "next_page_token": next_page_token,
            "stored_info_types": stored_info_types,
        }
        expected_response = dlp_pb2.ListStoredInfoTypesResponse(**expected_response)

        # Mock the API response
        channel = ChannelStub(responses=[expected_response])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_stored_info_types(parent)
        resources = list(paged_list_response)
        assert len(resources) == 1

        assert expected_response.stored_info_types[0] == resources[0]

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.ListStoredInfoTypesRequest(parent=parent)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_list_stored_info_types_exception(self):
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        parent = client.organization_location_path("[ORGANIZATION]", "[LOCATION]")

        paged_list_response = client.list_stored_info_types(parent)
        with pytest.raises(CustomException):
            list(paged_list_response)

    def test_delete_stored_info_type(self):
        channel = ChannelStub()
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup Request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        client.delete_stored_info_type(name)

        assert len(channel.requests) == 1
        expected_request = dlp_pb2.DeleteStoredInfoTypeRequest(name=name)
        actual_request = channel.requests[0][1]
        assert expected_request == actual_request

    def test_delete_stored_info_type_exception(self):
        # Mock the API response
        channel = ChannelStub(responses=[CustomException()])
        patch = mock.patch("google.api_core.grpc_helpers.create_channel")
        with patch as create_channel:
            create_channel.return_value = channel
            client = dlp_v2.DlpServiceClient()

        # Setup request
        name = client.organization_stored_info_type_path(
            "[ORGANIZATION]", "[STORED_INFO_TYPE]"
        )

        with pytest.raises(CustomException):
            client.delete_stored_info_type(name)



# -*- coding: utf-8 -*-
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import pytest

from google.cloud import dlp_v2
from google.cloud.dlp_v2 import enums
from google.cloud.dlp_v2.proto import dlp_pb2
from google.api_core import exceptions

from test_utils.vpcsc_config import vpcsc_config


_VPCSC_PROHIBITED_MESSAGE = "Request is prohibited by organization's policy"


@pytest.fixture(scope="module")
def client():
    return dlp_v2.DlpServiceClient()


@pytest.fixture(scope="module")
def name_inside(client):
    return client.project_path(vpcsc_config.project_inside)


@pytest.fixture(scope="module")
def name_outside(client):
    return client.project_path(vpcsc_config.project_outside)


@pytest.fixture(scope="module")
def content_item():
    return dlp_pb2.ContentItem(value="testing")


@pytest.fixture(scope="module")
def bytes_content_item():
    return dlp_pb2.ByteContentItem(data=b"DEADBEEF")


@vpcsc_config.skip_unless_inside_vpcsc
def test_inspect_content_inside(client, name_inside, content_item):
    client.inspect_content(name_inside, item=content_item)  # no perms issue


@vpcsc_config.skip_unless_inside_vpcsc
def test_inspect_content_outside(client, name_outside, content_item):
    with pytest.raises(exceptions.PermissionDenied) as exc:
        client.inspect_content(name_outside, item=content_item)

    assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@vpcsc_config.skip_unless_inside_vpcsc
def test_redact_image_inside(client, name_inside, bytes_content_item):
    with pytest.raises(exceptions.InvalidArgument):  # no perms issue
        client.redact_image(name_inside, byte_item=bytes_content_item)


@vpcsc_config.skip_unless_inside_vpcsc
def test_redact_image_outside(client, name_outside, bytes_content_item):
    with pytest.raises(exceptions.PermissionDenied) as exc:
        client.redact_image(name_outside, byte_item=bytes_content_item)

    assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@vpcsc_config.skip_unless_inside_vpcsc
def test_deidentify_content_inside(client, name_inside, content_item):
    with pytest.raises(exceptions.InvalidArgument):  # no perms issue
        client.deidentify_content(name_inside, item=content_item)


@vpcsc_config.skip_unless_inside_vpcsc
def test_deidentify_content_outside(client, name_outside, content_item):
    with pytest.raises(exceptions.PermissionDenied) as exc:
        client.deidentify_content(name_outside, item=content_item)

    assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@vpcsc_config.skip_unless_inside_vpcsc
def test_reidentify_content_inside(client, name_inside, content_item):
    with pytest.raises(exceptions.InvalidArgument):  # no perms issue
        client.reidentify_content(name_inside, item=content_item)


@vpcsc_config.skip_unless_inside_vpcsc
def test_reidentify_content_outside(client, name_outside, content_item):
    with pytest.raises(exceptions.PermissionDenied) as exc:
        client.reidentify_content(name_outside, item=content_item)

    assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@pytest.fixture(scope="module")
def inspect_template_path_inside(client):
    inspect_template_id = 1234567
    return client.project_inspect_template_path(
        vpcsc_config.project_inside, inspect_template_id
    )


@pytest.fixture(scope="module")
def inspect_template_path_outside(client):
    inspect_template_id = 1234567
    return client.project_inspect_template_path(
        vpcsc_config.project_outside, inspect_template_id
    )


@vpcsc_config.skip_unless_inside_vpcsc
class TestCRUDInspectTemplate(object):
    @staticmethod
    def test_create_inspect_template_inside(client, name_inside):
        client.create_inspect_template(name_inside)  # no perms issue

    @staticmethod
    def test_create_inspect_template_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.create_inspect_template(name_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_list_inspect_templates_inside(client, name_inside):
        list(client.list_inspect_templates(name_inside))

    @staticmethod
    def test_list_inspect_templates_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            list(client.list_inspect_templates(name_outside))

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_update_inspect_template_inside(client, inspect_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.update_inspect_template(inspect_template_path_inside)

    @staticmethod
    def test_update_inspect_template_outside(client, inspect_template_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.update_inspect_template(inspect_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_get_inspect_template_inside(client, inspect_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.get_inspect_template(inspect_template_path_inside)

    @staticmethod
    def test_get_inspect_template_outside(client, inspect_template_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.get_inspect_template(inspect_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_delete_inspect_template_inside(client, inspect_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.delete_inspect_template(inspect_template_path_inside)

    @staticmethod
    def test_delete_inspect_template_outside(client, inspect_template_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.delete_inspect_template(inspect_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@pytest.fixture(scope="module")
def deidentify_template_path_inside(client):
    deidentify_template_id = 1234567
    return client.project_deidentify_template_path(
        vpcsc_config.project_inside, deidentify_template_id
    )


@pytest.fixture(scope="module")
def deidentify_template_path_outside(client):
    deidentify_template_id = 1234567
    return client.project_deidentify_template_path(
        vpcsc_config.project_outside, deidentify_template_id
    )


@vpcsc_config.skip_unless_inside_vpcsc
class TestCRUDDeidentifyTemplate(object):
    @staticmethod
    def test_create_deidentify_template_inside(client, name_inside):
        client.create_deidentify_template(name_inside)

    @staticmethod
    def test_create_deidentify_template_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.create_deidentify_template(name_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_list_deidentify_templates_inside(client, name_inside):
        list(client.list_deidentify_templates(name_inside))

    @staticmethod
    def test_list_deidentify_templates_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            list(client.list_deidentify_templates(name_outside))

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_update_deidentify_template_inside(client, deidentify_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.update_deidentify_template(deidentify_template_path_inside)

    @staticmethod
    def test_update_deidentify_template_outside(
        client, deidentify_template_path_outside
    ):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.update_deidentify_template(deidentify_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_get_deidentify_template_inside(client, deidentify_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.get_deidentify_template(deidentify_template_path_inside)

    @staticmethod
    def test_get_deidentify_template_outside(client, deidentify_template_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.get_deidentify_template(deidentify_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_delete_deidentify_template_inside(client, deidentify_template_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.delete_deidentify_template(deidentify_template_path_inside)

    @staticmethod
    def test_delete_deidentify_template_outside(
        client, deidentify_template_path_outside
    ):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.delete_deidentify_template(deidentify_template_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@pytest.fixture(scope="module")
def job_path_inside(name_inside):
    job_id = 1234567
    return "{}/jobs/{}".format(name_inside, job_id)


@pytest.fixture(scope="module")
def job_path_outside(name_outside):
    job_id = 1234567
    return "{}/jobs/{}".format(name_outside, job_id)


@pytest.fixture(scope="module")
def inspect_job():
    from google.cloud.dlp_v2.proto.dlp_pb2 import InspectJobConfig

    return InspectJobConfig()


@vpcsc_config.skip_unless_inside_vpcsc
class TestCRUDDlpJob(object):
    @staticmethod
    def test_create_dlp_job_inside(client, name_inside, inspect_job):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.create_dlp_job(name_inside, inspect_job=inspect_job)

    @staticmethod
    def test_create_dlp_job_outside(client, name_outside, inspect_job):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.create_dlp_job(name_outside, inspect_job=inspect_job)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_list_dlp_jobs_inside(client, name_inside):
        list(client.list_dlp_jobs(name_inside))

    @staticmethod
    def test_list_dlp_jobs_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            list(client.list_dlp_jobs(name_outside))

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_get_dlp_job_inside(client, job_path_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.get_dlp_job(job_path_inside)

    @staticmethod
    def test_get_dlp_job_outside(client, job_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.get_dlp_job(job_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_delete_dlp_job_inside(client, job_path_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.delete_dlp_job(job_path_inside)

    @staticmethod
    def test_delete_dlp_job_outside(client, job_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.delete_dlp_job(job_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_cancel_dlp_job_inside(client, job_path_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.cancel_dlp_job(job_path_inside)

    @staticmethod
    def test_cancel_dlp_job_outside(client, job_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.cancel_dlp_job(job_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@pytest.fixture(scope="module")
def job_trigger_path_inside(client):
    job_trigger_id = 1234567
    return client.project_job_trigger_path(vpcsc_config.project_inside, job_trigger_id)


@pytest.fixture(scope="module")
def job_trigger_path_outside(client):
    job_trigger_id = 1234567
    return client.project_job_trigger_path(vpcsc_config.project_outside, job_trigger_id)


@vpcsc_config.skip_unless_inside_vpcsc
class TestCRUDJobTrigger(object):
    @staticmethod
    def test_create_job_trigger_inside(client, name_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.create_job_trigger(name_inside)

    @staticmethod
    def test_create_job_trigger_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.create_job_trigger(name_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_list_job_triggers_inside(client, name_inside):
        list(client.list_job_triggers(name_inside))

    @staticmethod
    def test_list_job_triggers_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            list(client.list_job_triggers(name_outside))

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_get_job_trigger_inside(client, job_trigger_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.get_job_trigger(job_trigger_path_inside)

    @staticmethod
    def test_get_job_trigger_outside(client, job_trigger_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.get_job_trigger(job_trigger_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_update_job_trigger_inside(client, job_trigger_path_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.update_job_trigger(job_trigger_path_inside)

    @staticmethod
    def test_update_job_trigger_outside(client, job_trigger_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.update_job_trigger(job_trigger_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_delete_job_trigger_inside(client, job_trigger_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.delete_job_trigger(job_trigger_path_inside)

    @staticmethod
    def test_delete_job_trigger_outside(client, job_trigger_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.delete_job_trigger(job_trigger_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message


@pytest.fixture(scope="module")
def stored_info_type_path_inside(client):
    stored_info_type_id = 1234567
    return client.project_stored_info_type_path(
        vpcsc_config.project_inside, stored_info_type_id
    )


@pytest.fixture(scope="module")
def stored_info_type_path_outside(client):
    stored_info_type_id = 1234567
    return client.project_stored_info_type_path(
        vpcsc_config.project_outside, stored_info_type_id
    )


@vpcsc_config.skip_unless_inside_vpcsc
class TestCRUDStoredInfoType(object):
    @staticmethod
    def test_create_stored_info_type_inside(client, name_inside):
        with pytest.raises(exceptions.InvalidArgument):  # no perms issue
            client.create_stored_info_type(name_inside)

    @staticmethod
    def test_create_stored_info_type_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.create_stored_info_type(name_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_list_stored_info_types_inside(client, name_inside):
        list(client.list_stored_info_types(name_inside))

    @staticmethod
    def test_list_stored_info_types_outside(client, name_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            list(client.list_stored_info_types(name_outside))

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_get_stored_info_type_inside(client, stored_info_type_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.get_stored_info_type(stored_info_type_path_inside)

    @staticmethod
    def test_get_stored_info_type_outside(client, stored_info_type_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.get_stored_info_type(stored_info_type_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_update_stored_info_type_inside(client, stored_info_type_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.update_stored_info_type(stored_info_type_path_inside)

    @staticmethod
    def test_update_stored_info_type_outside(client, stored_info_type_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.update_stored_info_type(stored_info_type_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message

    @staticmethod
    def test_delete_stored_info_type_inside(client, stored_info_type_path_inside):
        with pytest.raises(exceptions.NotFound):  # no perms issue
            client.delete_stored_info_type(stored_info_type_path_inside)

    @staticmethod
    def test_delete_stored_info_type_outside(client, stored_info_type_path_outside):
        with pytest.raises(exceptions.PermissionDenied) as exc:
            client.delete_stored_info_type(stored_info_type_path_outside)

        assert _VPCSC_PROHIBITED_MESSAGE in exc.value.message
